/****************************************************************************
 * UniPROG Universal programmer 
 * Chip database
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 *
 * 08/05/2001 - First version
 * 01/09/2003 - Added vendor indentify function
 *
 */

#ifndef CHIPDB_H
#define CHIPDB_H


#define SRAM     0
#define MASKROM  1
#define UVEPROM  2
#define OTPEPROM 3
#define EEPROM   4
#define FLASH    5

typedef const struct {
  char *name;                   // identifier
  unsigned int id;              // ID of device (for autotect)
  char *desc;                   // long description
  char *brand;                  // manofacturer
  int tech;                     // technology
  int pins;                     // number of pins
  unsigned long size;           // size in kb
  int nBits;                    // bits per addr
  int algorithm;                // program algorimth
  float vcc;                    // Vcc Voltage
  float vpp;                    // Vpp Voltage (only for EPROM)
  int nSectors;                 // Number of sectors (Only for FLASH)
  } CHIPREG;

int chipdbInit( void );
int chipdbGetNum( void );
CHIPREG *chipdbGet( int reg );
CHIPREG *chipdbFind( char *name );
CHIPREG *chipdbFindId( unsigned int id );
void chipdbPrintList( void );
const char *chipdbGetVendorName(unsigned char id);

#endif // CHIPDB_H

