/****************************************************************************
 * EEPROG Universal programmer 
 * Abstract programmer device manager          
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 *
 * 04/09/2003 - First version, updated programmers interface
 *              TODO: Create an flexible interface for bit algorithms 
 *
 */

#ifndef PROGDEV_H
#define PROGDEV_H


/* Programmer data structure */

struct prog_dev {
  struct prog_dev *next;
  char name[4];
  char description[32];
  int  refs;
  void *private_data;
  int  (* open) (struct prog_dev *);
  int  (* close)(struct prog_dev *);
  unsigned char  (* read_byte_addr)(struct prog_dev *, long );
  void  (* write_byte_addr)(struct prog_dev *, long, unsigned char );
  int  (* test_hardware)(struct prog_dev *, int );
  int  (* command)(int, void *, int, int); 
};

/* Public interface */ 
int    prgdev_init( void );
void   prgdev_list_devices( void );
struct prog_dev *prgdev_get_device( int );
int    prgdev_register( struct prog_dev * );
int    prgdev_select_device( const char *name );
struct prog_dev *prgdev_get_selected( void );


/* Programmer command set */ 

#define CMD_VCC  1 
#define CMD_VPP  2

/* Programmer error codes */

#define ERR_UNSUPPORTED    -1
#define ERR_NOHARDWARE     -2
#define ERR_INVALID_PARAMS -3
#define ERR_OK              0


#endif /* PROGDEV_H */
