/****************************************************************************
 * UniPROG Universal programmer 
 * Willem programmer support
 * 
 * Daniel Quintero <danielqg@infonegocio.com>
 *
 * 23/07/2003 - First version
 */


#define WILLEM_C

#include "stdhdr.h"
#include "progdev.h"
#include "portio.h"
#include "willem.h"

// Helpfull macros
#define uSleep(delay)   for(slp=0; slp<(10*delay); slp++);
int slp;


/* Programmer device structure */
struct prog_dev pd_willem;
  
/* Private variables */
BYTE     wilData;
BYTE     wilControl;
DWORD    wilAddress;
int      wilWaitDelay; 
unsigned wilCtrlPort;
unsigned wilDataPort;
unsigned wilStatusPort;


// Basic I/O Functions

int wilInit ( void ) {
  struct prog_dev *pd = &pd_willem;

  /* Init variables */
  wilData = 0x00;
  wilControl = 0xc;
  wilDataPort = 0x378;
  wilStatusPort = wilDataPort + 1;
  wilCtrlPort = wilDataPort + 2;
  wilAddress = 0xFFFFFF;

  memset(pd, 0, sizeof(*pd));
  strcpy(pd->name, "wil");
  strcpy(pd->description, "Willem eprommer");
  pd->open = wilOpen;
  pd->close = wilClose;
  pd->read_byte_addr = wilReadByteAddr;
  pd->write_byte_addr = wilWriteByteAddr;
  pd->test_hardware = wilCheckHard;
  pd->command = wilExecCommand;
  
  /* register device */
  prgdev_register( &pd_willem );

  return 1;
}


int wilOpen(struct prog_dev *pd ) {
  int err;

#ifdef __LINUX__
   if (ioperm(wilDataPort, 3, 1) == -1) {
   	printf("Error, i/o access denied for spp/epp\n");
   	return FALSE;
   	}
                                                                                                                             
   printf("Usermode i/o access granted\n");
#endif

  /* Power off, all lines off */
  read,adrlo,pgrlo,ppoff,powoff,clklo,clalo,assert_ctrl;assert_data;
  wilSetAddr(0x000000);
  
  /* Check for hardware present */
  err = wilCheckHard( pd, 0 );
  if (err == ERR_OK)
    pd->refs++;

  return err;
}

int wilClose( struct prog_dev *pd ) {
  /* Power off, all lines off */
  read,adrlo,pgrlo,ppoff,powoff,clklo,clalo,assert_ctrl;assert_data;
  wilSetAddr(0x000000);

  pd->refs--;
  return 1;
}


/* Programmer command interface */
int wilExecCommand( int cmd, void *ptr, int par1, int par2) {
  int err;

  switch (cmd) {
  case CMD_VCC : 
    if (par1 == 1)
      err = wilPowerOn();
    else if (par1 == 0)
      err = wilPowerDown();
    else
      err = ERR_INVALID_PARAMS;
    break;
 
  default:
    err = ERR_UNSUPPORTED;
  
  }
  return err;
}




// Read a byte from data bus
BYTE wilReadByte( void ){
     int b, w;


     OE_Low;                  /* S6=0, OE=0 data bus read, addr control  */
     /* Load shift register with data in D0..D7 */
     adrhi;assert_data;   /* D = 1, OE=0 shift register parallel load mode */
     clahi;assert_data;   /* cla hi, load parallel clock */
     adrlo;assert_data;   /* D = 0, OE=1 Shift register, serial mode */
     clalo;assert_data;   /* cla lo, serial clock low */
      
     /* Shift data to serial output (ACK) */
     for ( w=0, b=7; b>=0; --b) {
         w |= (inbit ? 0 : 1) << b; /* Get data from sr */
	 clahi;assert_data;             /* serial clock high */
	 clalo;assert_data;             /* serial clock low */
     }
     return w;
   }

/* Set chip address */
void wilSetAddr (long Addr) {
     int mask;

     //Addr = 0x555555;

     //if (Addr == wilAddress)  /* Check if address set yet */
     //  return;
     wilAddress = Addr;       /* New address */

     OE_Low;                  /* S6=0, OE=0 data bus triestate, addr control  */
     for (mask=(1 << 23); mask != 0; mask = mask >> 1){
       (Addr & mask) ? adrhi: adrlo;    /* out bit */
       clklo; assert_data;    /* shift clk low */
       clkhi; assert_data;    /* shift clk high */
   }
     adrlo; clklo; assert_data;      
}

void wilIncAddr( void )
   {
   wilAddress++;
   wilSetAddr(wilAddress);
   }

long  wilGetAddr( void )
   {
   return wilAddress;
   }


/* Power control */

int wilPowerDown(void)     // ARW: added proc to set all lines to low on
{                           //exit to avoid cmos latchup & allow chip removal
    read,adrlo,pgrlo,ppoff,powoff,clklo,clalo,assert_ctrl;assert_data;
    wilSetAddr(0x000000);

    return ERR_OK;
}

int wilPowerOn(void)        
{
    wilSetAddr(0x000000);    
    read,adrlo,pgrhi,ppoff,powoff,clklo,clalo,assert_ctrl;assert_data;
    waitms(10);
    VCC_On;
    waitms(100);

    return ERR_OK;
}

/* hardware test */

int wilCheckHard( struct prog_dev *pd, int mode ) {
  int result;
  int l,r;

  wilPowerOn();
  OE_Low;
  outhi;assert_data;waitms(10);
  r=einbit;
  outlo;assert_data;waitms(10);
  l=einbit;
  
  if(l==128 && r==0)  {
    result = ERR_OK;
    printf("Willem hardware present\n");
  }
  else  {
    result = ERR_NOHARDWARE;
    fprintf(stderr, "Willem hardware Error :  Check Power && connection");
  }
  wilPowerDown();        
  return result;
}


/* CPU Bus simulation routines for flash chips */

// Read a byte from Chip current address
unsigned char wilReadByteAddr( struct prog_dev *pd, long addr ) {
  unsigned char data;
  
  wilSetAddr(addr); /* Set Address */
  OE_Low;
  data = wilReadByte();
  return data;
}

// Write a byte to Chip specified address
void wilWriteByteAddr(struct prog_dev *pd, long addr, unsigned char val) {
  
  wilSetAddr(addr);   	      /* Set Address */
  OE_High;		      /* OE = 1 */
  OUTPORT(wilDataPort, val);  /* Put data on bus*/
  WE_Low;             	      /* Write Pulse */
  WE_High;
  OE_Low;		      /* OE = 0 */	
}

